package benefactoraddressvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/address"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateAddAddress(req param.BenefactorAddAddressRequest) (map[string]string, error) {
	const op = "benefactorvalidator.ValidateAddRequest"
	if err := validation.ValidateStruct(&req,

		validation.Field(&req.Address, validation.Required),

		validation.Field(&req.BenefactorID,
			validation.Required,
			validation.By(v.doesBenefactorExist)),

		validation.Field(&req.Lon,
			validation.Required),

		validation.Field(&req.Lat,
			validation.Required),

		validation.Field(&req.PostalCode,
			validation.Required,
		),
		validation.Field(&req.CityID,
			validation.Required,
			validation.By(v.doesCityExist)),

		validation.Field(&req.ProvinceID,
			validation.Required,
			validation.By(v.doesProvinceExist)),
	); err != nil {

		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
